Crate mogh_auth_server

Crate mogh_auth_server 

Modules

api
middleware
provider
rand
request_ip
user
validations
Default username / password validations. These can be overridden on AuthImpl.

Enums

RequestAuthentication

Traits

AuthImpl
This trait is implemented at the app level to support custom schemas, storage providers, and business logic.

Type Aliases

BoxAuthImpl
DynFuture